package com.ichaoj.ams.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ichaoj.ams.entity.AmsAirdropTask;
import com.ichaoj.ams.request.airdrop.CreateAirdropProject;
import com.ichaoj.ams.request.airdrop.PageProjectRequest;
import com.ichaoj.ams.request.airdrop.UpdateAirdropProject;
import com.ichaoj.ams.response.airdrop.AirdropProjectResponse;
import com.ichaoj.ams.response.airdrop.ProjectDetailResponse;
import com.ichaoj.ams.response.task.TaskResponse;
import com.ichaoj.ams.service.IAmsAirdropProjectService;
import com.ichaoj.ams.service.IAmsAirdropTaskService;
import com.ichaoj.common.annotation.AuthResource;
import com.ichaoj.common.exception.ErrorServiceException;
import com.ichaoj.common.model.PublicPage;
import com.ichaoj.common.model.PublicResult;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * <p>
 * 空投项目 前端控制器
 * </p>
 *
 * @author cjwen
 * @since 2023-04-25
 */
@RestController
@RequestMapping("/airdrop-project")
@Tag(name = "项目模块")
public class AirdropProjectController {

    @Resource
    private IAmsAirdropProjectService airdropProjectService;
    @Resource
    private IAmsAirdropTaskService taskService;

    @PostMapping
    @Operation(summary = "创建空投项目")
    @AuthResource(true)
    public PublicResult<Object> createAirdropProject(@RequestBody CreateAirdropProject createAirdropProject) {
        airdropProjectService.createAirdropProject(createAirdropProject);
        return PublicResult.success();
    }

    @PutMapping
    @Operation(summary = "修改空投项目")
    @AuthResource(true)
    public PublicResult<Object> updateAirdropProject(@RequestBody @Validated UpdateAirdropProject updateAirdropProject) {
        airdropProjectService.updateAirdropProject(updateAirdropProject);
        return PublicResult.success();
    }

    @DeleteMapping("/{projectId}")
    @Operation(summary = "删除空投项目")
    @AuthResource(true)
    public PublicResult<Object> deleteAirdropProject(@PathVariable String projectId) {
        airdropProjectService.removeById(projectId);
        return PublicResult.success();
    }

    @PostMapping("/page")
    @Operation(summary = "分页查询空投项目")
    @AuthResource
    public PublicResult<PublicPage<AirdropProjectResponse>> pageAirdropProject(@RequestBody PageProjectRequest projectRequest) {
        PublicPage<AirdropProjectResponse> result = airdropProjectService.pageAirdropProject(projectRequest);
        return PublicResult.success(result);
    }


    @GetMapping("/detail/{projectId}")
    @Operation(summary = "根据项目id查询详情")
    @AuthResource
    public PublicResult<ProjectDetailResponse> getByProjectId(@PathVariable String projectId) {
        if (StrUtil.isBlank(projectId)){
            throw new ErrorServiceException("projectId 不能为空");
        }
        ProjectDetailResponse project = BeanUtil.copyProperties(airdropProjectService.getById(projectId), ProjectDetailResponse.class);
        List<AmsAirdropTask> tasks = taskService.list(new LambdaQueryWrapper<AmsAirdropTask>().eq(AmsAirdropTask::getAirdropProjectId, project.getAmsProjectId()));
        List<TaskResponse> taskResponses = BeanUtil.copyToList(tasks, TaskResponse.class);
        project.setTasks(taskResponses);
        return PublicResult.success(project);
    }
}
